"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.removeItemFromTree = exports.getTreePosition = exports.getParent = exports.getItem = exports.flattenTree = exports.addItemToTree = void 0;
var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));
var _path = require("./path");
var _mutateTree = require("./mutateTree");
function _createForOfIteratorHelper(r, e) { var t = "undefined" != typeof Symbol && r[Symbol.iterator] || r["@@iterator"]; if (!t) { if (Array.isArray(r) || (t = _unsupportedIterableToArray(r)) || e && r && "number" == typeof r.length) { t && (r = t); var _n = 0, F = function F() {}; return { s: F, n: function n() { return _n >= r.length ? { done: !0 } : { done: !1, value: r[_n++] }; }, e: function e(r) { throw r; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var o, a = !0, u = !1; return { s: function s() { t = t.call(r); }, n: function n() { var r = t.next(); return a = r.done, r; }, e: function e(r) { u = !0, o = r; }, f: function f() { try { a || null == t.return || t.return(); } finally { if (u) throw o; } } }; }
function _unsupportedIterableToArray(r, a) { if (r) { if ("string" == typeof r) return _arrayLikeToArray(r, a); var t = {}.toString.call(r).slice(8, -1); return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray(r, a) : void 0; } }
function _arrayLikeToArray(r, a) { (null == a || a > r.length) && (a = r.length); for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e]; return n; }
/*
  Transforms tree structure into flat list of items for rendering purposes.
  We recursively go through all the elements and its children first on each level
 */
var flattenTree = exports.flattenTree = function flattenTree(tree) {
  var path = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  return tree.items[tree.rootId] ? tree.items[tree.rootId].children.reduce(function (accum, itemId, index) {
    // iterating through all the children on the given level
    var item = tree.items[itemId];
    var currentPath = [].concat((0, _toConsumableArray2.default)(path), [index]);
    // we create a flattened item for the current item
    var currentItem = createFlattenedItem(item, currentPath);
    // we flatten its children
    var children = flattenChildren(tree, item, currentPath);
    // append to the accumulator
    return [].concat((0, _toConsumableArray2.default)(accum), [currentItem], (0, _toConsumableArray2.default)(children));
  }, []) : [];
};

/*
  Constructs a new FlattenedItem
 */
var createFlattenedItem = function createFlattenedItem(item, currentPath) {
  return {
    item: item,
    path: currentPath
  };
};

/*
  Flatten the children of the given subtree
*/
var flattenChildren = function flattenChildren(tree, item, currentPath) {
  return item.isExpanded ? flattenTree({
    rootId: item.id,
    items: tree.items
  }, currentPath) : [];
};
var getItem = exports.getItem = function getItem(tree, path) {
  var cursor = tree.items[tree.rootId];
  var _iterator = _createForOfIteratorHelper(path),
    _step;
  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var i = _step.value;
      cursor = tree.items[cursor.children[i]];
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }
  return cursor;
};
var getParent = exports.getParent = function getParent(tree, path) {
  var parentPath = (0, _path.getParentPath)(path);
  return getItem(tree, parentPath);
};
var getTreePosition = exports.getTreePosition = function getTreePosition(tree, path) {
  var parent = getParent(tree, path);
  var index = (0, _path.getIndexAmongSiblings)(path);
  return {
    parentId: parent.id,
    index: index
  };
};
var hasLoadedChildren = function hasLoadedChildren(item) {
  return !!item.hasChildren && item.children.length > 0;
};
var isLeafItem = function isLeafItem(item) {
  return !item.hasChildren;
};
var removeItemFromTree = exports.removeItemFromTree = function removeItemFromTree(tree, position) {
  var sourceParent = tree.items[position.parentId];
  var newSourceChildren = (0, _toConsumableArray2.default)(sourceParent.children);
  var itemRemoved = newSourceChildren.splice(position.index, 1)[0];
  var newTree = (0, _mutateTree.mutateTree)(tree, position.parentId, {
    children: newSourceChildren,
    hasChildren: newSourceChildren.length > 0,
    isExpanded: newSourceChildren.length > 0 && sourceParent.isExpanded
  });
  return {
    tree: newTree,
    itemRemoved: itemRemoved
  };
};
var addItemToTree = exports.addItemToTree = function addItemToTree(tree, position, item) {
  var destinationParent = tree.items[position.parentId];
  var newDestinationChildren = (0, _toConsumableArray2.default)(destinationParent.children);
  if (typeof position.index === 'undefined') {
    if (hasLoadedChildren(destinationParent) || isLeafItem(destinationParent)) {
      newDestinationChildren.push(item);
    }
  } else {
    newDestinationChildren.splice(position.index, 0, item);
  }
  return (0, _mutateTree.mutateTree)(tree, position.parentId, {
    children: newDestinationChildren,
    hasChildren: true
  });
};